package cmd

import (
    "strings"
    u "nexus-client/utils"
)

// Dirty HACK:
// Commands whose Name starts with "!" are not real commands, but headers.
// Section headers are just section separators that are printed in the help output.

type Command interface {
    GetName() string
    GetDesc() []string
    PrintUsage()
    Run(args []string) int   // Returns 0 if the Command was successful.
}

func Help(commands []Command) {
    padding := 20
    for _, cmd := range commands {
        cmdStr := cmd.GetName()
        if (isHeader(cmd)) {
            u.PrintlnErr(cmdStr[1:])
        } else {
            u.PrintErr(cmdStr)
            pad := strings.Repeat(" ", padding-len(cmd.GetName()))
            for i, desc := range cmd.GetDesc() {
                if i > 0 {
                    u.PrintErr(strings.Repeat(" ", len(cmdStr)))
                }
                u.PrintlnErr(pad+desc)
            }
        }
        u.PrintlnErr("")
    }
}

// Returns true if the specified command exists.
func Match(name string, cmdList []Command) (bool, Command) {
    for _, cmd := range cmdList {
        if (!isHeader(cmd)) {
            if name == cmd.GetName() {
                return true, cmd
            }
        }
    }
    return false, nil
}

func isHeader(cmd Command) bool {
    return cmd.GetName()[0:1] == "!"
}
