package params

import (
    "strconv"
)

type Version struct {
    Major int      `json:"major" validate:"required,gte=0"`
    Minor int      `json:"minor" validate:"required,gte=0"`
    Bugfix int     `json:"bugfix" validate:"required,gte=0"`
}

func NewVersion(major, minor, bugfix int) Version {
    return Version{major, minor, bugfix}
}

// // Creates a version from a string fromatted as: "x.y.z"
// func NewVersionFromString(s string) (Version, error) {
//     f := strings.Split(s, ".")
//     major, _ := strconv.Atoi(f[0])
//     minor, _ := strconv.Atoi(f[1])
//     bugfix, _ := strconv.Atoi(f[2])
//     return Version{major, minor, bugfix}, nil
// }

func (v *Version)String() string {
    return strconv.Itoa(v.Major)+"."+strconv.Itoa(v.Minor)+"."+strconv.Itoa(v.Bugfix)
}

func (v *Version)StringWithoutBugfix() string {
    return strconv.Itoa(v.Major)+"."+strconv.Itoa(v.Minor)+".x"
}

// Returns true if v is older than candidate
func (v *Version)IsOlder(candidate *Version) bool {
    v1 := v.getMajorMinor()
    v2 := candidate.getMajorMinor()
    if v1 > v2 {
        return false
    } else if v1 < v2 {
        return true
    } else {
        bugfix1 := v.getBugfix()
        bugfix2 := candidate.getBugfix()
        return bugfix1 < bugfix2
    }
}

// Returns true if v is compatible with candidate
func (v *Version)IsCompatible(candidate *Version) bool {
    v1 := v.getMajorMinor()
    v2 := candidate.getMajorMinor()
    return v1 == v2
}

func (v *Version)getMajorMinorAsString() string {
    return strconv.Itoa(v.Major)+strconv.Itoa(v.Minor)
}

func (v *Version)getMajorMinor() int {
    n, _ := strconv.Atoi(v.getMajorMinorAsString())
    return n
}

func (v *Version)getBugfix() int {
    return v.Bugfix
}
